@@ -27,7 +27,11 @@ import datetime
 from datetime import timedelta
 import numpy as np
 from pytz import timezone
+from agstream.decorators import timeit_info
+import logging
+logger = logging.getLogger(__name__)
 
+logging.basicConfig(level=logging.DEBUG)
 
 """
   Object principal de connection avec les service Agriscoep.
@@ -59,18 +63,11 @@ class AgspSession(object):
         self.connector = AgspConnecteur(server=server)
         self.set_debug(False)
         self.sessionId = 0
-        self.station = False
         self.timezoneName = timezoneName
         self.tz = timezone(self.timezoneName)
         self.useInternalsSensors = False;
         self.ms_resolution = use_ms_resolution
   
-    def set_debug(self, value):
-        """
-        Set the debug flag. More verbose
-        """
-        self.debug = value
-        self.connector.set_debug(value)
 
     """
     login()
@@ -97,11 +94,11 @@ class AgspSession(object):
         :return: True if login OK or False
         """
         if isinstance(login_p, str) and isinstance(password_p, str):
-## removing space causing error
+        ## removing space causing error
             login_p=login_p.replace(" ", "")
             password_p=password_p.replace(" ", "") 
         else :
-            print(u"Erreur lors de la connection, login ou password doivent etre une string")
+            logger.error(u"Erreur lors de la connection, login ou password doivent etre une string")
             return False
         
 
@@ -110,15 +107,15 @@ class AgspSession(object):
         if status == True:
             # reinitialise
             self.agribases = list()
-            print(login_p + u" logging OK.")
+            logger.info(login_p + u" logging OK.")
             if updateAgribaseInfo == True:
                 self.__refreshAgribases()
         if self.debug == True:
             if status == True:
-                print(login_p + u" logging OK.")
+                logger.info(login_p + u" logging OK.")
 
             else:
-                print(u"Erreur de connection pour " + login_p + ".")
+                logger.error(u"Erreur de connection pour " + login_p + ".")
         self.status = status
         self.sessionId = sessionId
         return status
@@ -154,7 +151,7 @@ class AgspSession(object):
     """ 
     Retourne un dataframe pour l'agribase pour la periode demandÃ©e 
     """
-
+    #@timeit_info
     def getAgribaseDataframe(self, agribase_p, from_p=None, to_p=None,index_by_sensor_id = False):
         """
         getAgribaseDataframe
@@ -178,7 +175,7 @@ class AgspSession(object):
         
         :return: `Pandas <https://pandas.pydata.org/>`__  Dataframe with data
         """
-
+        from_p,to_p=self.set_timezone_if_naive(from_p,to_p)
         frame = self.getAgribaseDataframeDeep(agribase_p.serialNumber,from_p, to_p)
 
         if index_by_sensor_id == False : # so index with sensor name
@@ -215,6 +212,8 @@ class AgspSession(object):
         
         :return: `Pandas <https://pandas.pydata.org/>`__  Dataframe with data
         """
+        from_p,to_p=self.set_timezone_if_naive(from_p,to_p)
+
         frame = pd.DataFrame()
         frame.tz_convert(self.tz)
 
@@ -223,13 +222,18 @@ class AgspSession(object):
         if from_p == None:
             from_p = to_p - timedelta(days=3)
         
-        result_dict = self.__loadAgribaseDataFlat(agribase_serial_number,from_p,to_p) 
+        # get the data from the api
+        result_dict =self.connector.getAgribaseAllSensorsData( agribase_serial_number,  self._totimestamp(from_p), self._totimestamp(to_p)) 
+        
+        # change columns header by sensor id; 
         for sensor_id in result_dict.keys():
             dates,values = result_dict[sensor_id]
             label= "%d"%sensor_id
+            # convert data to pandas dataframe
             df = self.__convertDataToPandasFrame(dates, values, label)
+            df = self.__check_datagram_interval_limits(df, from_p,to_p)
+
             if df is not None and len(df) > 0:
-                # print frame.head()
                 frame = pd.concat([frame, df], axis=1)
         
         if frame is not None:
@@ -238,51 +242,11 @@ class AgspSession(object):
 
    
 
-    
-    """ 
-    Retourne un dataframe pour l'agribase sur l'instant donnée [+/- 1 seconde]
-    """
-
-    def getAgribaseDataframeRealTime(self, agribaseSerialNumber, atDate=None):
-        """
-        getAgribaseDataframeRealTime
-        --------------------
-        Return a `Pandas <https://pandas.pydata.org/>`__  Dataframe fill with data
-        generated by the Agribase.
-        
-                
-        :param agribaseSerialNumber: Agribase serialNumber.
-        :param atDate: date of the data
-        
-        :return: `Pandas <https://pandas.pydata.org/>`__  Dataframe with data
-        """
-        frame = pd.DataFrame()
-        frame.tz_convert(self.tz)
-
-        agribase = self.getAgribase(agribaseSerialNumber)
-        from_p = atDate - timedelta(seconds=1);
-        to_p = atDate + timedelta(seconds=1);
-        result_dict = self.__loadAgribaseDataFlat(agribase.serial_number,from_p,to_p)
-        for sensor_id in result_dict.keys():
-            data_tuples = result_dict[sensor_id]
-            sensor = agribase.getSensorByAgspSensorId(sensor_id)
-
-        for sens in agribase.sensors:
-            df = self.getSensorDataframe(sens, from_p, to_p)
-            if df is not None and len(df) > 0:
-                # print frame.head()
-                frame = pd.concat([frame, df], axis=1)
-        if frame is not None:
-            frame = frame.tz_convert(self.tz)
-        return frame
-
-
-
     """ 
     Retourne un dataframe pour le capteur pour la periode demandée 
     """
 
-    def getSensorDataframe(self, sensor, from_p=None, to_p=None):
+    def getSensorDataframe(self, sensor, from_p=None, to_p=None,index_by_sensor_id = False):
         """
         getSensorDataframe
         --------------------
@@ -305,8 +269,19 @@ class AgspSession(object):
         
         :return: `Pandas <https://pandas.pydata.org/>`__  Dataframe with data
         """
-        return self.getSensorDataframeDeep(sensor.agspSensorId, sensor.name, from_p, to_p)
+        if sensor.isVirtualDriver == False :
+            df = self.getSensorDataframeDeep(sensor.agspSensorId, sensor.name, from_p, to_p)
+        else :
+            df= self.getVirtualSensorDataframeDeep(sensor.agspSensorId,sensor.agribase_serial_number, from_p, to_p)
 
+        if index_by_sensor_id == False : # so index with sensor name
+            renaming_dict = dict()
+            for col in df.columns :
+                renaming_dict[col] = sensor.name
+            df.rename(columns=renaming_dict,inplace=True)
+        return df
+    
+    
     def describe(self):
         """
         Return some information about the session.
@@ -319,67 +294,40 @@ class AgspSession(object):
         for abse in self.agribases:
             print("    - " + str(abse.name) + "")
 
-    """
-    Refresh information
-    """
-
-    def __refreshAgribases(self):
-        json = self.connector.getAgribases(showInternalSensors=self.useInternalsSensors)
-        self.agribases = list()
-        for tmpjson in json["agribases"]:
-            abse = Agribase()
-            abse.loadFromJson(tmpjson)
-            # if abse.intervalInSeconds == -1 :
-            # abse.intervalInSeconds = self.connector.getAgribaseIntervaleInSeconds(abse.serialNumber)
-            self.agribases.append(abse)
-        return self.agribases
-
+   
+    
+    
     def getSensorDataframeDeep(self, sensorid, sensor_name, from_p=None, to_p=None):
         if to_p == None:
             to_p = self.tz.localize(datetime.datetime.now())
         if from_p == None:
             from_p = to_p - timedelta(days=3)
-        date, values = self.__loadSensorDataFlat(sensorid, from_p, to_p)
-        dataFrame = self.__convertDataToPandasFrame(date, values, sensor_name)
-        if dataFrame is not None:
-            dataFrame = dataFrame.tz_convert(self.tz)
-        return dataFrame
+        date, values = self.connector.getSensorData(sensorid,  self._totimestamp(from_p), self._totimestamp(to_p))
+        df = self.__convertDataToPandasFrame(date, values, sensorid)
+        df = self.__check_datagram_interval_limits(df, from_p,to_p)
 
-    def ___getAgribaseTypeName(self, serialNumber):
-        url = "http://jsonmaint.agriscope.fr/tools/CHECK/agbs.php?sn=%d" % serialNumber
-        json = self.connector.executeJsonRequest(url)
-        returnv = -1
-        if "agbsType" in json:
-            returnv = json["agbsType"]
-        return returnv
-
-    def __getDataframe(self, datesArray_p, valuesArray_p, limitFrom, limitTo):
-        # en python les tableaux sont des list.
-        newFrame = self.convertDataToPandasFrame(
-            datesArray_p, valuesArray_p, self.datasource.getSmallName()
-        )
-        newFrame.sort_index(inplace=True)
-        if len(newFrame) > 0:
-            # On borne la plage....
-            # Bug du serveur ariscope... En effet parfois il renvoie des dates hors de l'intervalle demandée (par exemple 1992)
-            # On limit l'effet, en 'coupant' les index dont les dates ne sont pas dans l'intervalle demane
-            currentFirst = newFrame.index[0]
-            currentLast = newFrame.index[len(newFrame) - 1]
-            if limitFrom > currentFirst:
-                newFrame = newFrame[limitFrom:currentLast]
-
-            if limitTo < currentLast:
-                newFrame = newFrame[currentFirst:limitTo]
-
-        if len(newFrame) > 0:
-            # print_full(newFrame)
-            self.rawPandaDataFrame = pd.concat([self.rawPandaDataFrame, newFrame])
-        return self.rawPandaDataFrame
+        if df is not None:
+            df = df.tz_convert(self.tz)
+        return df       
+            
+    #@timeit_info        
+    def getVirtualSensorDataframeDeep(self, sensorid,  agribase_sn, from_p=None, to_p=None):
+        if to_p == None:
+            to_p = self.tz.localize(datetime.datetime.now())
+        if from_p == None:
+            from_p = to_p - timedelta(days=3)
+        date, values = self.connector.get_virtual_datasource_data( agribase_sn,sensorid,  self._totimestamp(from_p), self._totimestamp(to_p))
+   
+        df = self.__convertDataToPandasFrame(date, values, sensorid)
+        df = self.__check_datagram_interval_limits(df, from_p,to_p)
+        if df is not None:
+            df = df.tz_convert(self.tz)
+        return df
 
+   
     def __convertDataToPandasFrame(self, datesArray_p, valuesArray_p, label):
         freshDates = []
         freshValues = []
-        ##print 'len(dates) : %d, len(values) : %d' % (len(datesArray_p), len(valuesArray_p))
         if len(datesArray_p) > 0:
             for i in range(len(datesArray_p)):
 
@@ -391,24 +339,51 @@ class AgspSession(object):
             # Remove freshValue
         freshDates = []
         freshValues = []
-        return None
+        return pd.DataFrame()  
+    
+    
+    def __check_datagram_interval_limits(self, df, from_p, to_p) :
+        df.sort_index(inplace=True)
+        if len(df) > 0:
+            # On borne la plage....
+            # Bug du serveur ariscope... En effet parfois il renvoie des dates hors de l'intervalle demandée (par exemple 1992)
+            # On limit l'effet, en 'coupant' les index dont les dates ne sont pas dans l'intervalle demane
+            currentFirst = df.index[0]
+            currentLast = df.index[len(df) - 1]
+            if from_p > currentFirst:
+                df = df[from_p:currentLast]
 
-    def __loadSensorData(self, sensor=None, from_p=None, to_p=None):
-        return self.__loadSensorDataFlat(
-            sensor.agspSensorId,
-            self._totimestamp(from_p),
-            self._totimestamp(to_p),
-        )
+            if to_p < currentLast:
+                df = df[currentFirst:to_p]
+        return df  
+    
+    def __refreshAgribases(self):
+        json = self.connector.getAgribases(showInternalSensors=self.useInternalsSensors)
+        self.agribases = list()
+        for tmpjson in json["agribases"]:
+            abse = Agribase()
+            abse.loadFromJson(tmpjson)
+            # if abse.intervalInSeconds == -1 :
+            # abse.intervalInSeconds = self.connector.getAgribaseIntervaleInSeconds(abse.serialNumber)
+            self.agribases.append(abse)
+        return self.agribases
 
-    def __loadSensorDataFlat(self, sensorId=None, from_p=None, to_p=None):
-        return self.connector.getSensorData(
-            sensorId,  self._totimestamp(from_p), self._totimestamp(to_p))
-        
-    def __loadAgribaseDataFlat(self, agribase_sn=None, from_p=None, to_p=None):
-            return self.connector.getAgribaseAllSensorsData(
-            agribase_sn,  self._totimestamp(from_p), self._totimestamp(to_p))
-            
-        
+
+    def __getAgribaseTypeName(self, serialNumber):
+        url = "http://jsonmaint.agriscope.fr/tools/CHECK/agbs.php?sn=%d" % serialNumber
+        json = self.connector.executeJsonRequest(url)
+        returnv = -1
+        if "agbsType" in json:
+            returnv = json["agbsType"]
+        return returnv
+
+    def set_timezone_if_naive (self,from_p=None,to_p=None) :
+        if (from_p is not None) and  (from_p.tzinfo  is None or from_p.tzinfo.utcoffset(from_p) is  None) :
+            from_p = self.tz.localize(from_p)
+        if (to_p is not None) and  (to_p.tzinfo  is  None or to_p.tzinfo.utcoffset(to_p) is  None) :
+            to_p = self.tz.localize(to_p)
+        return from_p,to_p
+    
     def _totimestamp(self, dt_obj):
         """
         Args:
@@ -444,11 +419,8 @@ class AgspSession(object):
         # ----------------------------------------------------------------------
         # Traceback (most recent call last):
         #  File "C:\Users\guillaume\Documents\Developpement\django\trunk\datatooling\pandas\tests\agspUniversTests.py", line 37, in test_firstUnivers
-        # print unixtimeStamp
         if unixtimeStamp < 0:
             unixtimeStamp = 1
-        # print "unixtimestamp=" + unicode(unixtimeStamp)
-        #print("unixtimestamp %f, %.2f, %.4f"% (unixtimeStamp,old_div(unixtimeStamp, 1000),unixtimeStamp/ 1000) )
         if self.ms_resolution == True :
             returnv = pytz.utc.localize(
             datetime.datetime.utcfromtimestamp(unixtimeStamp/ 1000)
@@ -457,16 +429,22 @@ class AgspSession(object):
             returnv = pytz.utc.localize(
             datetime.datetime.utcfromtimestamp(old_div(unixtimeStamp, 1000))
             )
-        #print (returnv.strftime("%d/%m/%y %HH%mm%Ss%f"))
-        # print unicode(returnv)
-        # print "%s" % returnv.year
-        # if (returnv.year == 1992) :
-
-        # print "%d %s" % (unixtimeStamp, unicode(returnv))
         return returnv
 
+    def set_debug(self, value):
+        """
+        Set the debug flag. More verbose
+        """
+        self.debug = value
+        self.connector.set_debug(value)
+        
+    def remove_any_timezone_info(self, inputdf) :
+        df = inputdf.copy()
+        df.index = df.index.tz_localize(None)
+        for col in df.select_dtypes(['datetimetz']).columns:
+            df[col] = df[col].dt.tz_convert(None)
+        return df
 
-def _touchLogin(self, login_p, password_p):
-    status, sessionId = self.connector.login(login_p, password_p)
-    self.status = status
-    self.sessionId = sessionId
+
+    
+    
